@@ -173,7 +173,7 @@ def get_switchable_as_dense(network, component, attr, snapshots=None, inds=None)
         pd.DataFrame(np.repeat([df.loc[fixed_i, attr].values], len(snapshots), axis=0),
                      index=snapshots, columns=fixed_i),
         pnl[attr].loc[snapshots, varying_i]
-    ], axis=1, sort=False).reindex(columns=index))
+    ], axis=1, sort=False).reindex(index=snapshots, columns=index))
 
 def get_switchable_as_iter(network, component, attr, snapshots, inds=None):
     """
@@ -312,6 +312,18 @@ def get_non_extendable_i(n, c):
     return n.df(c)[lambda ds: ~ds[nominal_attrs[c] + '_extendable']].index
 
 
+def get_committable_i(n, c):
+    """
+    Getter function. Get the index of commitable elements of a given
+    component.
+    """
+    if "committable" not in n.df(c):
+        idx = pd.Index([])
+    else:
+        idx = n.df(c)[lambda ds: ds['committable']].index
+    return idx.rename(f'{c}-com')
+
+
 def get_active_assets(n, c, investment_period):
     """
     Getter function. Get True values for elements of component c which are active
