@@ -75,8 +75,11 @@ class ImporterCSV(Importer):
         fn = os.path.join(self.csv_folder_name, "snapshots.csv")
         if not os.path.isfile(fn): return None
         df = pd.read_csv(fn, index_col=0, encoding=self.encoding, parse_dates=True)
+        # backwards-compatibility: level "snapshot" was rename to "timestep"
         if "snapshot" in df:
             df["snapshot"] = pd.to_datetime(df.snapshot)
+        if "timestep" in df:
+            df["timestep"] = pd.to_datetime(df.timestep)
         return df
 
     def get_investment_periods(self):
@@ -327,7 +330,7 @@ def _export_to_exporter(network, exporter, basename, export_standard_types=False
 
     #now export snapshots
     if isinstance(network.snapshot_weightings.index, pd.MultiIndex):
-        network.snapshot_weightings.index.rename(["period", "snapshot"], inplace=True)
+        network.snapshot_weightings.index.rename(["period", "timestep"], inplace=True)
     else:
         network.snapshot_weightings.index.rename("snapshot", inplace=True)
     snapshots = network.snapshot_weightings.reset_index()
@@ -597,10 +600,11 @@ def _import_from_importer(network, importer, basename, skip_time=False):
     ##https://docs.python.org/3/tutorial/datastructures.html#comparing-sequences-and-other-types
     if pypsa_version is None or pypsa_version < current_pypsa_version:
         logger.warning(dedent("""
-                Importing PyPSA from older version of PyPSA than current version {}.
-                Please read the release notes at https://pypsa.org/doc/release_notes.html
+                Importing PyPSA from older version of PyPSA than current version.
+                Please read the release notes at https://pypsa.readthedocs.io/en/latest/release_notes.html
                 carefully to prepare your network for import.
-        """).format(network.pypsa_version))
+                Currently used PyPSA version {}, imported network file PyPSA version {}.
+        """).format(current_pypsa_version, pypsa_version))
 
     importer.pypsa_version = pypsa_version
     importer.current_pypsa_version = current_pypsa_version
@@ -609,9 +613,9 @@ def _import_from_importer(network, importer, basename, skip_time=False):
     df = importer.get_snapshots()
 
     if df is not None:
-
         # check if imported snapshots have MultiIndex
-        snapshot_levels = set(["period", "snapshot"]).intersection(df.columns)
+        # backwards-compatibility: level "snapshot" was rename to "timestep"
+        snapshot_levels = set(["period", "timestep", "snapshot"]).intersection(df.columns)
         if snapshot_levels:
             df.set_index(sorted(snapshot_levels), inplace=True)
         network.set_snapshots(df.index)
@@ -732,6 +736,7 @@ def import_components_from_dataframe(network, dataframe, cls_name):
         logger.error("Error, new components for {} are not unique".format(cls_name))
         return
 
+    new_df.index.name = cls_name
     setattr(network, network.components[cls_name]["list_name"], new_df)
 
     #now deal with time-dependent properties
@@ -781,6 +786,8 @@ def import_series_from_dataframe(network, dataframe, cls_name, attr):
     pnl = network.pnl(cls_name)
     list_name = network.components[cls_name]["list_name"]
 
+    dataframe.columns.name = cls_name
+    dataframe.index.name = 'snapshot'
     diff = dataframe.columns.difference(df.index)
     if len(diff) > 0:
         logger.warning(f"Components {diff} for attribute {attr} of {cls_name} "
