@@ -50,6 +50,7 @@ def _as_snapshots(network, snapshots):
     if not isinstance(snapshots, pd.MultiIndex):
         snapshots = pd.Index(snapshots)
     assert snapshots.isin(network.snapshots).all()
+    snapshots.name = 'snapshot'
     return snapshots
 
 
@@ -1086,7 +1087,7 @@ def find_cycles(sub_network, weight='x_pu'):
 
     #reduce to a non-multi-graph for cycles with > 2 edges
     mgraph = sub_network.graph(weight=weight, inf_weight=False)
-    graph = nx.OrderedGraph(mgraph)
+    graph = nx.Graph(mgraph)
 
     cycles = nx.cycle_basis(graph)
 
